export type Reject<E> = (err: E) => void;
export type Resolve<T> = (t: T) => void;
export type Cancel = () => void;
export type Computation<E, T> = (reject: Reject<E>, resolve: Resolve<T>) => Cancel;
export declare class Task<E, T> {
    /**
     * A Task that is always successful. Resolves to `t`.
     */
    static succeed<E, T>(t: T): Task<E, T>;
    /**
     * A Task that always fails. Rejects with `err`.
     */
    static fail<E, T>(err: E): Task<E, T>;
    /**
     * Converts a function that returns a Promise into a Task. Promise based tasks
     * can't be cancelled.
     */
    static fromPromise<E, T>(fn: () => Promise<T>): Task<E, T>;
    /**
     * Creates a new task that will run a series of tasks in parallel. If any
     * of the tasks reject, then all other results are discarded. If all tasks
     * resolve, then the an array of results is returned.
     *
     * This is comparable to Promise.all
     *
     * Implementation is based on https://github.com/futurize/parallel-future
     */
    static all<E, T>(ts: Array<Task<E, T>>): Task<E, T[]>;
    /**
     * Creates a new Task from an Array of Tasks. When forked, all tasks are
     * forked in parallel. The first task to complete (either rejected or resolved)
     * is preserved. All other results are discarded.
     *
     * This could be used for a simple timeout mechanism. If the timeout rejects
     * before the fetch completes, you'll get a timeout error.
     *
     *     new Task([longFetchTask, timeoutTask])
     */
    static race<E, T>(tasks: ReadonlyArray<Task<E, T>>): Task<E, T>;
    /**
     * `loop` returns a higher-order task forks the "inner" task until the inner task
     * succeeds. Retries forking the task on the interval number the same as using
     * setTimeout.
     *
     * The cancel function returned when the loop is forked can be called to cancel the
     * loop. Cancelling the loop causes the task to never resolve.
     */
    static loop<E, T>(interval: number, task: Task<E, T>): Task<never, T>;
    private fn;
    constructor(computation: Computation<E, T>);
    /**
     * Run the task. If the task fails, the reject function is called, and passed
     * the error. If the task succeeds, then the resolve function is called with
     * the task result.
     *
     * The fork function also returns a Cancel function. Calling the cancel
     * function will abort the task, provided that the task actually supports
     * cancelling. `succeed` and `fail`, for example, return the cancel function,
     * but it is a No Op, since those tasks resolve immediately.
     */
    fork(reject: Reject<E>, resolve: Resolve<T>): Cancel;
    /**
     * Run the task wrapped in a Promise and return the Promise. Once a task is
     * resolved as a Promise, the Promise API is used to interact with it.
     *
     * Each time `resolve` is called on a Task, a new Promise is generated. All
     * of the side-effects of executing the Task will fire.
     */
    resolve(): Promise<T>;
    /**
     * Execute a function in the context of a successful task
     */
    map<A>(f: (t: T) => A): Task<E, A>;
    /**
     * An alias for `map`
     */
    and<A>(f: (t: T) => A): Task<E, A>;
    /**
     * Execute a Task in the context of a successful task. Flatten the result.
     */
    andThen<A>(f: (t: T) => Task<E, A>): Task<E, A>;
    /**
     * Execute a Promise in the context of a successful task, as though it were
     * a Task. Flatten the result and convert to a Task.
     *
     * In theory, it means that you could take a browser api like `fetch`, which
     * is promises all the way down, and chain it right into a normal task chain.
     *
     * For example:
     *
     *     Task.succeed('https://jsonplaceholder.typicode.com/posts/1')
     *       .andThenP(fetch)
     *       .andThenP(result => result.json())
     *       .andThen(obj => someDecoder.decodeAny(obj).cata(Err: Task.fail, Ok: Task.succeed))
     *       .fork(
     *         err => `You died: ${err}`,
     *         someThing => doSomethingAwesomeHereWithThis(someThing)
     *       )
     */
    andThenP<A>(f: (t: T) => Promise<A>): Task<E, A>;
    /**
     * Execute a Task in the context of a failed task. Flatten the result.
     */
    orElse<X>(f: (err: E) => Task<X, T>): Task<X, T>;
    /**
     * Execute a function in the context of a failed task.
     */
    mapError<X>(f: (err: E) => X): Task<X, T>;
    /**
     * Assign encapsulates a pattern of building up an object (or a scope) from
     * a series of Task results. Without `assign`, you would need to nest a series
     * of `andThen` calls to build a shared javascript scope if you needs to
     * combine results. That pattern can become indistinguishable from callback
     * hell. Using `assign`, you can flatten out your call chain, while maintaining
     * type safety.
     *
     * The idea and the base implementation came from this blog post:
     * https://medium.com/@dhruvrajvanshi/simulating-haskells-do-notation-in-typescript-e48a9501751c
     */
    assign<K extends string, A>(k: K, other: Task<E, A> | ((t: T) => Task<E, A>)): Task<E, T & {
        [k in K]: A;
    }>;
    /**
     * Inject a side-effectual function into a task call chain. Task themselves are the
     * appropriate way to handle side-effects that you care about, but sometimes you
     * may want to do an _fire-and-forget_ side effect. The most common example of this
     * is performing a logging the current value of a Task.
     *
     *     Task.succeed('hi')
     *       .do(v => console.log(v))
     *       .andThen(doSomethingWithGreeting)
     *
     * `do` will only run in the context of a successful task.
     */
    do(fn: (a: T) => void): Task<E, T>;
    /**
     * Inject a side-effectual function into a task call chain. Task themselves are the
     * appropriate way to handle side-effects that you care about, but sometimes you
     * may want to do an _fire-and-forget_ side effect. The most common example of this
     * is performing a logging the current value of a Task.
     *
     *     Task.fail('oops')
     *       .elseDo(v => console.log(v))
     *       .orElse(doSomethingWithError)
     *
     * `elseDo` will only run in the context of a failed task.
     */
    elseDo(fn: (err: E) => void): Task<E, T>;
}
