"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Task = void 0;
const deps_js_1 = require("./deps.js");
class Task {
    /**
     * A Task that is always successful. Resolves to `t`.
     */
    static succeed(t) {
        return new Task((_reject, resolve) => {
            resolve(t);
            return deps_js_1.noop;
        });
    }
    /**
     * A Task that always fails. Rejects with `err`.
     */
    static fail(err) {
        return new Task((reject, _resolve) => {
            reject(err);
            return deps_js_1.noop;
        });
    }
    /**
     * Converts a function that returns a Promise into a Task. Promise based tasks
     * can't be cancelled.
     */
    static fromPromise(fn) {
        return new Task((reject, resolve) => {
            fn().then(resolve, reject);
            return deps_js_1.noop;
        });
    }
    /**
     * Creates a new task that will run a series of tasks in parallel. If any
     * of the tasks reject, then all other results are discarded. If all tasks
     * resolve, then the an array of results is returned.
     *
     * This is comparable to Promise.all
     *
     * Implementation is based on https://github.com/futurize/parallel-future
     */
    static all(ts) {
        const length = ts.length;
        if (length === 0) {
            return Task.succeed([]);
        }
        return new Task((reject, resolve) => {
            let resolved = 0;
            const results = [];
            const cancels = {};
            const resolveIdx = (idx) => (result) => {
                resolved = resolved + 1;
                results[idx] = result;
                cancels[idx] = deps_js_1.noop;
                if (resolved === length) {
                    resolve(results);
                }
            };
            for (let i = 0; i < length; i++) {
                cancels[i] = ts[i].fork(reject, resolveIdx(i));
            }
            return () => {
                Object.entries(cancels).forEach(([_, cancel]) => cancel());
            };
        });
    }
    /**
     * Creates a new Task from an Array of Tasks. When forked, all tasks are
     * forked in parallel. The first task to complete (either rejected or resolved)
     * is preserved. All other results are discarded.
     *
     * This could be used for a simple timeout mechanism. If the timeout rejects
     * before the fetch completes, you'll get a timeout error.
     *
     *     new Task([longFetchTask, timeoutTask])
     */
    static race(tasks) {
        return (0, deps_js_1.fromEmpty)(tasks).cata({
            Nothing: () => new Task((0, deps_js_1.always)(deps_js_1.noop)),
            Just: () => new Task((reject, resolve) => {
                let status = 'waiting';
                const cancels = [];
                const cancelAll = () => {
                    let cancel;
                    while ((cancel = cancels.shift()))
                        cancel();
                };
                const cancelAllIfNotStillWaiting = () => {
                    switch (status) {
                        case 'waiting':
                            break;
                        case 'rejected':
                        case 'resolved':
                            cancelAll();
                            break;
                    }
                };
                const resolveAndCancelAll = (t) => {
                    switch (status) {
                        case 'waiting':
                            status = 'resolved';
                            resolve(t);
                            cancelAll();
                            break;
                        case 'resolved':
                        case 'rejected':
                            break;
                    }
                };
                const rejectAndCancelAll = (e) => {
                    switch (status) {
                        case 'waiting':
                            status = 'rejected';
                            reject(e);
                            cancelAll();
                            break;
                        case 'resolved':
                        case 'rejected':
                            break;
                    }
                };
                const forkTaskIfStillWaiting = (task) => {
                    switch (status) {
                        case 'waiting': {
                            const cancel = task.fork(rejectAndCancelAll, resolveAndCancelAll);
                            cancels.push(cancel);
                            break;
                        }
                        case 'rejected':
                        case 'resolved':
                            break;
                    }
                };
                for (let i = 0; i < tasks.length; i++) {
                    forkTaskIfStillWaiting(tasks[i]);
                }
                cancelAllIfNotStillWaiting();
                return () => cancelAll();
            }),
        });
    }
    /**
     * `loop` returns a higher-order task forks the "inner" task until the inner task
     * succeeds. Retries forking the task on the interval number the same as using
     * setTimeout.
     *
     * The cancel function returned when the loop is forked can be called to cancel the
     * loop. Cancelling the loop causes the task to never resolve.
     */
    static loop(interval, task) {
        return new Task((_, resolve) => {
            let timeout;
            let internalCancel;
            const loop = () => {
                internalCancel = task.fork(() => {
                    timeout = setTimeout(loop, interval);
                }, (result) => resolve(result));
            };
            loop();
            return () => {
                clearTimeout(timeout);
                internalCancel && internalCancel();
            };
        });
    }
    constructor(computation) {
        Object.defineProperty(this, "fn", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.fn = computation;
    }
    /**
     * Run the task. If the task fails, the reject function is called, and passed
     * the error. If the task succeeds, then the resolve function is called with
     * the task result.
     *
     * The fork function also returns a Cancel function. Calling the cancel
     * function will abort the task, provided that the task actually supports
     * cancelling. `succeed` and `fail`, for example, return the cancel function,
     * but it is a No Op, since those tasks resolve immediately.
     */
    fork(reject, resolve) {
        return this.fn(reject, resolve);
    }
    /**
     * Run the task wrapped in a Promise and return the Promise. Once a task is
     * resolved as a Promise, the Promise API is used to interact with it.
     *
     * Each time `resolve` is called on a Task, a new Promise is generated. All
     * of the side-effects of executing the Task will fire.
     */
    resolve() {
        return new Promise((resolve, reject) => {
            this.fn(reject, resolve);
        });
    }
    /**
     * Execute a function in the context of a successful task
     */
    map(f) {
        return new Task((reject, resolve) => {
            return this.fn((err) => reject(err), (a) => resolve(f(a)));
        });
    }
    /**
     * An alias for `map`
     */
    and(f) {
        return this.map(f);
    }
    /**
     * Execute a Task in the context of a successful task. Flatten the result.
     */
    andThen(f) {
        return new Task((reject, resolve) => {
            let innerCancel;
            const outerCancel = this.fn((err) => reject(err), (a) => {
                innerCancel = f(a).fork(reject, resolve);
            });
            return () => {
                if (innerCancel) {
                    innerCancel();
                }
                else {
                    outerCancel();
                }
            };
        });
    }
    /**
     * Execute a Promise in the context of a successful task, as though it were
     * a Task. Flatten the result and convert to a Task.
     *
     * In theory, it means that you could take a browser api like `fetch`, which
     * is promises all the way down, and chain it right into a normal task chain.
     *
     * For example:
     *
     *     Task.succeed('https://jsonplaceholder.typicode.com/posts/1')
     *       .andThenP(fetch)
     *       .andThenP(result => result.json())
     *       .andThen(obj => someDecoder.decodeAny(obj).cata(Err: Task.fail, Ok: Task.succeed))
     *       .fork(
     *         err => `You died: ${err}`,
     *         someThing => doSomethingAwesomeHereWithThis(someThing)
     *       )
     */
    andThenP(f) {
        return new Task((reject, resolve) => {
            return this.fn((err) => reject(err), (a) => f(a).then(resolve, reject));
        });
    }
    /**
     * Execute a Task in the context of a failed task. Flatten the result.
     */
    orElse(f) {
        return new Task((reject, resolve) => {
            let innerCancel;
            const outerCancel = this.fn((x) => {
                innerCancel = f(x).fork(reject, resolve);
            }, (t) => resolve(t));
            return () => {
                if (innerCancel) {
                    innerCancel();
                }
                else {
                    outerCancel();
                }
            };
        });
    }
    /**
     * Execute a function in the context of a failed task.
     */
    mapError(f) {
        return new Task((reject, resolve) => {
            return this.fn((e) => reject(f(e)), (t) => resolve(t));
        });
    }
    /**
     * Assign encapsulates a pattern of building up an object (or a scope) from
     * a series of Task results. Without `assign`, you would need to nest a series
     * of `andThen` calls to build a shared javascript scope if you needs to
     * combine results. That pattern can become indistinguishable from callback
     * hell. Using `assign`, you can flatten out your call chain, while maintaining
     * type safety.
     *
     * The idea and the base implementation came from this blog post:
     * https://medium.com/@dhruvrajvanshi/simulating-haskells-do-notation-in-typescript-e48a9501751c
     */
    assign(k, other) {
        return this.andThen((t) => {
            const task = typeof other === 'function' ? other(t) : other;
            return task.map((a) => ({
                ...Object(t),
                [k.toString()]: a,
            }));
        });
    }
    /**
     * Inject a side-effectual function into a task call chain. Task themselves are the
     * appropriate way to handle side-effects that you care about, but sometimes you
     * may want to do an _fire-and-forget_ side effect. The most common example of this
     * is performing a logging the current value of a Task.
     *
     *     Task.succeed('hi')
     *       .do(v => console.log(v))
     *       .andThen(doSomethingWithGreeting)
     *
     * `do` will only run in the context of a successful task.
     */
    do(fn) {
        return this.map((v) => {
            fn(v);
            return v;
        });
    }
    /**
     * Inject a side-effectual function into a task call chain. Task themselves are the
     * appropriate way to handle side-effects that you care about, but sometimes you
     * may want to do an _fire-and-forget_ side effect. The most common example of this
     * is performing a logging the current value of a Task.
     *
     *     Task.fail('oops')
     *       .elseDo(v => console.log(v))
     *       .orElse(doSomethingWithError)
     *
     * `elseDo` will only run in the context of a failed task.
     */
    elseDo(fn) {
        return this.mapError((err) => {
            fn(err);
            return err;
        });
    }
}
exports.Task = Task;
